"""
Get an initial ranking for a set of queries. The ranking may come
from an .inRank file or from a bag-of-words ranker (ranked and
unranked boolean, Indri, BM25).
"""

# Copyright (c) 2025, Carnegie Mellon University.  All Rights Reserved.

import itertools

from collections import OrderedDict

import Util

from Idx import Idx
from QryParser import QryParser
from Results import Results
from RetrievalModelUnrankedBoolean import RetrievalModelUnrankedBoolean

class Ranker:
    """
    Get an initial ranking for a set of queries. The ranking may
    come from an .inRank file or from a bag-of-words ranker (ranked
    and unranked boolean, Indri, BM25).
    """


    # -------------- Methods (alphabetical) ---------------- #

    def __init__(self, parameters):
        self._model = None
        self._inRank_path = None
        self._max_results = 1000       		# default

        if 'outputLength' in parameters:
            self._max_results = parameters['outputLength']

        if 'retrievalAlgorithm' not in parameters:
            raise AttributeError('Missing parameter: retrievalAlgorithm')

        if parameters['retrievalAlgorithm'] == 'UnrankedBoolean':
            self._model = RetrievalModelUnrankedBoolean(parameters)
        else:
            raise AttributeError('Unknown retrievalAlgorithm: ' \
                                 f'{parameters["retrievalAlgorithm"]}')


    def get_rankings(self, queries):
        """
        Get a list of rankings for a set of queries. Each ranking is
        a list of (score, externalId) tuples.

        queries: A dict of {query_id:query_string}.
        """
        if self._model is not None:
            return(self.get_rankings_bow(queries))
        elif self._inRank_path is not None:
            return(Util.read_rankings(self._inRank_path))
        else:
            raise Exception('Error: Ranker does not know how to rank')
                        

    def get_rankings_bow(self, queries):
        """
        Get a list of rankings for a set of queries. Each ranking is
        a list of (score, externalId) tuples.
        
        queries: A dict of {query_id:query_string}.
        """
        results = {}
        
        for qid, qString in queries.items():
            # Prepare to evaluate a query
            print(f'{qid}: {qString}')
            qString = f'{self._model.defaultQrySop}({qString})'
            q = QryParser.getQuery(qString)
            print(f'    ==> {str(q)}')
            q.initialize(self._model)
            results_qid = Results(self._max_results)

            # Evaluate the query. Each pass of the loop finds
            # one matching document.
            while(q.docIteratorHasMatch(self._model)):
                docid = q.docIteratorGetMatch()
                score = q.getScore(self._model)
                q.docIteratorAdvancePast(docid)
                results_qid.add(docid, score)

            results[qid] = results_qid.get_ranking()

        return(results)
