"""
Create, access, and manipulate document score lists.
"""

# Copyright (c) 2025, Carnegie Mellon University.  All Rights Reserved.


from Idx import Idx

class Results:
    """
    A Results object stores search results and returns the top n.
    """

    class Entry:
        """
        A utility class to create an <externalDocid, score> object
        for the _results list.
        """

        def __init__(self, score, externalId):
            self.externalId = externalId
            self.score = score

        def __lt__(self, other):
            return(self.score < other.score or
                   (self.score == other.score and
                    self.externalId > other.externalId))

        def __str__(self):
            return(f'{self.externalId} {self.score}')


    # -------------- Methods (alphabetical) ---------------- #

    def __init__(self, n):
        """Create an empty list that can store results."""
        self._max_size = n
        self._results = []


    def __len__(self):
        return(len(self._results))


    def add(self, internalId, score):
        """
        Add a document to the results (unsorted).
        internalId: An internal document id.
        score: The document score.
        """
        externalId = Idx.getExternalDocid(internalId)
        self._results.append(self.Entry(score, externalId))
        

    def get_ranking(self):
        """
        Get a ranked list of results in (score, externalId) order.
        """
        results_qid = [(r.score, r.externalId) for r in self._results]
        results_qid.sort(key=lambda r: (-r[0], r[1]))
        return(results_qid[0:self._max_size])


